@@ -112,7 +112,12 @@ on_session_started (GdmSession       *session,
         username = gdm_session_direct_get_username (slave->priv->session);
         gdm_slave_run_script (GDM_SLAVE (slave), GDMCONFDIR "/PreSession", username);
 
-        /* FIXME: should we do something here? */
+        /* FIXME: should we do something here?
+         * Note that error return status from PreSession script should
+         * be ignored in the case of a X-GDM-BypassXsession session, which can
+         * be checked by calling:
+         * gdm_session_direct_bypasses_xsession (session)
+         */
 }
 
 static void
@@ -174,7 +179,7 @@ reset_session (GdmSimpleSlave *slave)
 {
         destroy_session (slave);
         create_new_session (slave);
-        gdm_session_open (GDM_SESSION (slave->priv->session));
+        gdm_session_start_conversation (GDM_SESSION (slave->priv->session));
 }
 
 static gboolean
@@ -217,7 +222,7 @@ on_session_setup_failed (GdmSession     *session,
 {
         if (slave->priv->greeter_server != NULL) {
                 gdm_greeter_server_problem (slave->priv->greeter_server,
-                                            _("Unable to initialize login system"));
+                                           message != NULL ? message:  _("Unable to initialize login system"));
         }
 
         destroy_session (slave);
@@ -253,7 +258,7 @@ on_session_authentication_failed (GdmSession     *session,
 {
         if (slave->priv->greeter_server != NULL) {
                 gdm_greeter_server_problem (slave->priv->greeter_server,
-                                            _("Unable to authenticate user"));
+                                            message != NULL ? message : _("Unable to authenticate user"));
         }
         destroy_session (slave);
         queue_greeter_reset (slave);
@@ -307,7 +312,7 @@ on_session_authorization_failed (GdmSession     *session,
 {
         if (slave->priv->greeter_server != NULL) {
                 gdm_greeter_server_problem (slave->priv->greeter_server,
-                                            _("Unable to authorize user"));
+                                           message != NULL ? message :  _("Unable to authorize user"));
         }
 
         destroy_session (slave);
@@ -334,6 +339,8 @@ try_migrate_session (GdmSimpleSlave *slave)
 static void
 stop_greeter (GdmSimpleSlave *slave)
 {
+        char *username;
+
         g_debug ("GdmSimpleSlave: Stopping greeter");
 
         if (slave->priv->greeter == NULL) {
@@ -342,7 +349,11 @@ stop_greeter (GdmSimpleSlave *slave)
         }
 
         /* Run the PostLogin script. gdmslave suspends until script has terminated */
-        gdm_slave_run_script (GDM_SLAVE (slave), GDMCONFDIR "/PostLogin", GDM_USERNAME);
+        username = gdm_session_direct_get_username (slave->priv->session);
+
+        if (username != NULL) {
+                gdm_slave_run_script (GDM_SLAVE (slave), GDMCONFDIR "/PostLogin", username);
+        }
 
         gdm_welcome_session_stop (GDM_WELCOME_SESSION (slave->priv->greeter));
         gdm_greeter_server_stop (slave->priv->greeter_server);
@@ -407,7 +418,7 @@ static void
 on_session_accredited (GdmSession     *session,
                        GdmSimpleSlave *slave)
 {
-        queue_start_session (slave);
+        gdm_session_open_session (session);
 }
 
 static void
@@ -425,8 +436,14 @@ on_session_accreditation_failed (GdmSession     *session,
            accreditation fails */
         if (! migrated) {
                 if (slave->priv->greeter_server != NULL) {
+                        const char *problem;
+                        if (message) {
+                                problem = message;
+                        } else {
+                                problem = _("Unable to establish credentials");
+                        }
                         gdm_greeter_server_problem (slave->priv->greeter_server,
-                                                    _("Unable establish credentials"));
+                                                    problem);
                 }
         }
 
@@ -439,6 +456,27 @@ on_session_accreditation_failed (GdmSession     *session,
         queue_greeter_reset (slave);
 }
 
+static void
+on_session_opened (GdmSession     *session,
+                   GdmSimpleSlave *slave)
+{
+        queue_start_session (slave);
+}
+
+static void
+on_session_open_failed (GdmSession     *session,
+                        const char     *message,
+                        GdmSimpleSlave *slave)
+{
+        if (slave->priv->greeter_server != NULL) {
+                gdm_greeter_server_problem (slave->priv->greeter_server,
+                                            _("Unable to open session"));
+        }
+
+        destroy_session (slave);
+        queue_greeter_reset (slave);
+}
+
 static void
 on_session_info (GdmSession     *session,
                  const char     *text,
@@ -479,15 +517,15 @@ on_session_secret_info_query (GdmSession     *session,
 }
 
 static void
-on_session_opened (GdmSession     *session,
-                   GdmSimpleSlave *slave)
+on_session_conversation_started (GdmSession     *session,
+                                 GdmSimpleSlave *slave)
 {
         gboolean res;
         gboolean enabled;
         char    *username;
         int      delay;
 
-        g_debug ("GdmSimpleSlave: session opened");
+        g_debug ("GdmSimpleSlave: session conversation started");
         if (slave->priv->greeter_server != NULL) {
                 res = gdm_greeter_server_ready (slave->priv->greeter_server);
                 if (! res) {
@@ -598,8 +636,8 @@ create_new_session (GdmSimpleSlave *slave)
         g_free (display_hostname);
 
         g_signal_connect (slave->priv->session,
-                          "opened",
-                          G_CALLBACK (on_session_opened),
+                          "conversation-started",
+                          G_CALLBACK (on_session_conversation_started),
                           slave);
         g_signal_connect (slave->priv->session,
                           "setup-complete",
@@ -641,6 +679,14 @@ create_new_session (GdmSimpleSlave *slave)
                           "accreditation-failed",
                           G_CALLBACK (on_session_accreditation_failed),
                           slave);
+        g_signal_connect (slave->priv->session,
+                          "session-opened",
+                          G_CALLBACK (on_session_opened),
+                          slave);
+        g_signal_connect (slave->priv->session,
+                          "session-open-failed",
+                          G_CALLBACK (on_session_open_failed),
+                          slave);
         g_signal_connect (slave->priv->session,
                           "info",
                           G_CALLBACK (on_session_info),
@@ -816,7 +862,7 @@ on_greeter_connected (GdmGreeterServer *greeter_server,
 
         g_debug ("GdmSimpleSlave: Greeter connected");
 
-        gdm_session_open (GDM_SESSION (slave->priv->session));
+        gdm_session_start_conversation (GDM_SESSION (slave->priv->session));
 
         g_object_get (slave,
                       "display-is-local", &display_is_local,
@@ -1033,7 +1079,7 @@ static void
 on_server_ready (GdmServer      *server,
                  GdmSimpleSlave *slave)
 {
-        g_timeout_add (500, (GSourceFunc)idle_connect_to_display, slave);
+        g_idle_add ((GSourceFunc)idle_connect_to_display, slave);
 }
 
 static void
